@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2020, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifndef BOB_H__
 #define BOB_H__
 
@@ -34,12 +42,13 @@ namespace client
 	const char BOB_COMMAND_INPORT[] = "inport";
 	const char BOB_COMMAND_QUIET[] = "quiet";
 	const char BOB_COMMAND_LOOKUP[] = "lookup";
+	const char BOB_COMMAND_LOOKUP_LOCAL[] = "lookuplocal";
 	const char BOB_COMMAND_CLEAR[] = "clear";
 	const char BOB_COMMAND_LIST[] = "list";
 	const char BOB_COMMAND_OPTION[] = "option";
 	const char BOB_COMMAND_STATUS[] = "status";
 	const char BOB_COMMAND_HELP[] = "help";
-	
+
 	const char BOB_HELP_ZAP[] = "zap - Shuts down BOB.";
 	const char BOB_HELP_QUIT[] = "quit - Quits this session with BOB.";
 	const char BOB_HELP_START[] = "start - Starts the current nicknamed tunnel.";
@@ -54,7 +63,7 @@ namespace client
 	const char BOB_HELP_OUTPORT[] = "outport <PORT_NUMBER> - Set the outbound port that nickname contacts.";
 	const char BOB_HELP_INHOST[] = "inhost <HOSTNAME|IP> - Set the inbound hostname or IP.";
 	const char BOB_HELP_INPORT[] = "inport <PORT_NUMBER> - Set the inbound port number nickname listens on.";
-	const char BOB_HELP_QUIET[] = "quiet <True|False> - Wether to send the incoming destination.";
+	const char BOB_HELP_QUIET[] = "quiet <True|False> - Whether to send the incoming destination.";
 	const char BOB_HELP_LOOKUP[] = "lookup <I2P_HOSTNAME> - Look up an I2P hostname.";
 	const char BOB_HELP_CLEAR[] = "clear - Clear the current nickname out of the list.";
 	const char BOB_HELP_LIST[] = "list - List all tunnels.";
@@ -75,15 +84,15 @@ namespace client
 
 	class BOBI2PInboundTunnel: public BOBI2PTunnel
 	{
-			struct AddressReceiver
-			{
-				std::shared_ptr<boost::asio::ip::tcp::socket> socket;
-				char buffer[BOB_COMMAND_BUFFER_SIZE + 1]; // for destination base64 address
-				uint8_t * data; // pointer to buffer
-				size_t dataLen, bufferOffset;
+		struct AddressReceiver
+		{
+			std::shared_ptr<boost::asio::ip::tcp::socket> socket;
+			char buffer[BOB_COMMAND_BUFFER_SIZE + 1]; // for destination base64 address
+			uint8_t * data; // pointer to buffer
+			size_t dataLen, bufferOffset;
 
-				AddressReceiver (): data (nullptr), dataLen (0), bufferOffset (0) {};
-			};
+			AddressReceiver (): data (nullptr), dataLen (0), bufferOffset (0) {};
+		};
 
 		public:
 
@@ -115,7 +124,7 @@ namespace client
 	{
 		public:
 
-			 BOBI2POutboundTunnel (const std::string& outhost, int port, std::shared_ptr<ClientDestination> localDestination, bool quiet);
+			BOBI2POutboundTunnel (const std::string& outhost, int port, std::shared_ptr<ClientDestination> localDestination, bool quiet);
 
 			void Start ();
 			void Stop ();
@@ -154,6 +163,7 @@ namespace client
 			int GetInPort() const { return m_InPort; }
 			int GetOutPort() const { return m_OutPort; }
 			bool GetQuiet() const { return m_Quiet; }
+			bool IsRunning() const { return m_IsRunning; }
 			const i2p::data::PrivateKeys& GetKeys () const { return m_LocalDestination->GetPrivateKeys (); };
 			std::shared_ptr<ClientDestination> GetLocalDestination () const { return m_LocalDestination; };
 
@@ -162,11 +172,12 @@ namespace client
 			std::shared_ptr<ClientDestination> m_LocalDestination;
 			BOBI2POutboundTunnel * m_OutboundTunnel;
 			BOBI2PInboundTunnel * m_InboundTunnel;
-			
+
 			std::string m_Nickname;
 			std::string m_InHost, m_OutHost;
 			int m_InPort, m_OutPort;
 			bool m_Quiet;
+			bool m_IsRunning;
 	};
 
 	class BOBCommandChannel;
@@ -198,6 +209,7 @@ namespace client
 			void InportCommandHandler (const char * operand, size_t len);
 			void QuietCommandHandler (const char * operand, size_t len);
 			void LookupCommandHandler (const char * operand, size_t len);
+			void LookupLocalCommandHandler (const char * operand, size_t len);
 			void ClearCommandHandler (const char * operand, size_t len);
 			void ListCommandHandler (const char * operand, size_t len);
 			void OptionCommandHandler (const char * operand, size_t len);
@@ -215,14 +227,14 @@ namespace client
 			void SendReplyOK (const char * msg = nullptr);
 			void SendReplyError (const char * msg);
 			void SendRaw (const char * data);
-			
+
 			void BuildStatusLine(bool currentTunnel, BOBDestination *destination, std::string &out);
 
 		private:
 
 			BOBCommandChannel& m_Owner;
 			boost::asio::ip::tcp::socket m_Socket;
-            boost::asio::streambuf m_ReceiveBuffer, m_SendBuffer;
+			boost::asio::streambuf m_ReceiveBuffer, m_SendBuffer;
 			bool m_IsOpen, m_IsQuiet, m_IsActive;
 			std::string m_Nickname, m_InHost, m_OutHost;
 			int m_InPort, m_OutPort;
@@ -269,4 +281,3 @@ namespace client
 }
 
 #endif
-
