@@ -1,3 +1,11 @@
+/*
+* Copyright (c) 2013-2023, The PurpleI2P Project
+*
+* This file is part of Purple i2pd project and licensed under BSD3
+*
+* See full license text in LICENSE file at top of project tree
+*/
+
 #ifndef TRANSPORT_SESSION_H__
 #define TRANSPORT_SESSION_H__
 
@@ -16,6 +24,10 @@ namespace i2p
 {
 namespace transport
 {
+	const size_t IPV4_HEADER_SIZE = 20;
+	const size_t IPV6_HEADER_SIZE = 40;
+	const size_t UDP_HEADER_SIZE = 8;
+
 	class SignedData
 	{
 		public:
@@ -25,6 +37,12 @@ namespace transport
 			{
 				m_Stream << other.m_Stream.rdbuf ();
 			}
+
+			void Reset ()
+			{
+				m_Stream.str("");
+			}
+
 			void Insert (const uint8_t * buf, size_t len)
 			{
 				m_Stream.write ((char *)buf, len);
@@ -56,11 +74,13 @@ namespace transport
 		public:
 
 			TransportSession (std::shared_ptr<const i2p::data::RouterInfo> router, int terminationTimeout):
-				m_DHKeysPair (nullptr), m_NumSentBytes (0), m_NumReceivedBytes (0), m_IsOutgoing (router), m_TerminationTimeout (terminationTimeout),
+				m_NumSentBytes (0), m_NumReceivedBytes (0), m_SendQueueSize (0),
+				m_IsOutgoing (router), m_TerminationTimeout (terminationTimeout),
 				m_LastActivityTimestamp (i2p::util::GetSecondsSinceEpoch ())
 			{
 				if (router)
 					m_RemoteIdentity = router->GetRouterIdentity ();
+				m_CreationTime = m_LastActivityTimestamp;
 			}
 
 			virtual ~TransportSession () {};
@@ -68,39 +88,57 @@ namespace transport
 
 			std::string GetIdentHashBase64() const { return m_RemoteIdentity ? m_RemoteIdentity->GetIdentHash().ToBase64() : ""; }
 
-			std::shared_ptr<const i2p::data::IdentityEx> GetRemoteIdentity () 
+			std::shared_ptr<const i2p::data::IdentityEx> GetRemoteIdentity ()
 			{
 				std::lock_guard<std::mutex> l(m_RemoteIdentityMutex);
-				return m_RemoteIdentity; 
+				return m_RemoteIdentity;
 			}
-			void SetRemoteIdentity (std::shared_ptr<const i2p::data::IdentityEx> ident) 
+			void SetRemoteIdentity (std::shared_ptr<const i2p::data::IdentityEx> ident)
 			{
 				std::lock_guard<std::mutex> l(m_RemoteIdentityMutex);
-				m_RemoteIdentity = ident; 
+				m_RemoteIdentity = ident;
 			}
 
 			size_t GetNumSentBytes () const { return m_NumSentBytes; };
 			size_t GetNumReceivedBytes () const { return m_NumReceivedBytes; };
+			size_t GetSendQueueSize () const { return m_SendQueueSize; };
 			bool IsOutgoing () const { return m_IsOutgoing; };
 
 			int GetTerminationTimeout () const { return m_TerminationTimeout; };
 			void SetTerminationTimeout (int terminationTimeout) { m_TerminationTimeout = terminationTimeout; };
 			bool IsTerminationTimeoutExpired (uint64_t ts) const
-			{ return ts >= m_LastActivityTimestamp + GetTerminationTimeout (); };
+			{
+				return ts >= m_LastActivityTimestamp + GetTerminationTimeout () ||
+					ts + GetTerminationTimeout () < m_LastActivityTimestamp;
+			};
 
-			virtual void SendLocalRouterInfo () { SendI2NPMessages ({ CreateDatabaseStoreMsg () }); };
+			uint32_t GetCreationTime () const { return m_CreationTime; };
+			void SetCreationTime (uint32_t ts) { m_CreationTime = ts; }; // for introducers
+
+			virtual uint32_t GetRelayTag () const { return 0; };
+			virtual void SendLocalRouterInfo (bool update = false) { SendI2NPMessages ({ CreateDatabaseStoreMsg () }); };
 			virtual void SendI2NPMessages (const std::vector<std::shared_ptr<I2NPMessage> >& msgs) = 0;
+			virtual bool IsEstablished () const = 0;
 
 		protected:
 
 			std::shared_ptr<const i2p::data::IdentityEx> m_RemoteIdentity;
 			mutable std::mutex m_RemoteIdentityMutex;
-			std::shared_ptr<i2p::crypto::DHKeys> m_DHKeysPair; // X - for client and Y - for server
-			size_t m_NumSentBytes, m_NumReceivedBytes;
+			size_t m_NumSentBytes, m_NumReceivedBytes, m_SendQueueSize;
 			bool m_IsOutgoing;
 			int m_TerminationTimeout;
 			uint64_t m_LastActivityTimestamp;
+			uint32_t m_CreationTime; // seconds since epoch
 	};
+
+	// SOCKS5 proxy
+	const uint8_t SOCKS5_VER = 0x05;
+	const uint8_t SOCKS5_CMD_CONNECT = 0x01;
+	const uint8_t SOCKS5_CMD_UDP_ASSOCIATE = 0x03;
+	const uint8_t SOCKS5_ATYP_IPV4 = 0x01;
+	const uint8_t SOCKS5_ATYP_IPV6 = 0x04;
+	const size_t SOCKS5_UDP_IPV4_REQUEST_HEADER_SIZE = 10;
+	const size_t SOCKS5_UDP_IPV6_REQUEST_HEADER_SIZE = 22;
 }
 }
 
